@@ -34,12 +34,13 @@
 //
 //
 //
-
+// Corect GraphTool file
 //[Rcpp::depends(RcppArmadillo)]
 #include <iostream>
 #include <vector>
 #include <set>
 #include <queue>
+
 #include <RcppArmadillo.h>
 #include <map>
 using namespace std;
@@ -200,19 +201,14 @@ private:
                 return this->shortestFather;
             }//tell others who is your father
             
-            bool setDistance(double dis, Node* targetFather){
-                if (dis > this->distance) {
-                    return false;
-                }
-                else if (dis < this->distance){
-                    this->distance = dis;
-                    shortestFatherList.clear();
-                    setFather(targetFather);
-                }
-                shortestFatherList.push_back(targetFather);
-                if (targetFather->getName() < this->whoisYourFather()->getName())
-                    setFather(targetFather);
-                return true;
+            int compDistance(double dis, Node* targetFather){
+              if (dis - this->distance > std::numeric_limits<double>::epsilon()) {
+                return 0;
+              }
+              else if (abs(dis - this->distance) > std::numeric_limits<double>::epsilon()){
+                return 2;
+              }
+              return 1;
             }//check if the new node is shorter than the current node, if it is
             //update the current father node to new node
             
@@ -258,7 +254,7 @@ private:
         private:
             bool reverse;
         public:
-            compareNode(const bool& reverse = true){
+            compareNode(const bool& reverse = false){
                 this->reverse = reverse;
             }// prepare for reverse, which means make the map rank from large to small
             bool operator()(const Node* const &a, const Node* const &b) const{
@@ -317,6 +313,9 @@ private:
                         }
                     }
                 }
+                /*sort(listofNode[i]->neighbor.begin(), listofNode[i]->neighbor.end(), [](const std::pair<Node*,double> &left, const std::pair<Node*,double> &right) {
+                  return left.second < right.second;
+                });*/
             }//add the neighbor to the node
         }
         
@@ -346,7 +345,7 @@ private:
                     }
                 }
                 
-                for (int j = 0; j < reachedNodeFirstRound.size(); j++) {
+                for (long unsigned int j = 0; j < reachedNodeFirstRound.size(); j++) {
                     q = reachedNodeFirstRound[j]->neighbor.begin();
                     for (; q != reachedNodeFirstRound[j]->neighbor.end(); q++) {
                         if (q->first->getDegree() >= reachedNodeFirstRound[j]->getDegree()) {
@@ -385,38 +384,45 @@ private:
             startNode->setDistance(0);
             startNode->changeState();
             startNode->changeState();
-            //dealMap.push(listofNode[start]);
+            // dealMap.push(listofNode[start]);
             // dealMap.push(startNode);
             nodeQueue.insert(startNode);
             
             vector<Node*> reachedNode;
             
             while (!nodeQueue.empty()) {
-                // Node* top = dealMap.top();
-                // dealMap.pop();
-                Node* top = *nodeQueue.begin();
-                nodeQueue.erase(nodeQueue.begin());
-                
-                std::vector<std::pair<Node*, double> >::iterator q = top->neighbor.begin();
-                for (; q != top->neighbor.end(); q++) {
-                    if ((*q).first->askState(2)) continue;
-                    double tempDist = (*q).first->getDistance();
-                    if ((*q).first->setDistance((*q).second + top->getDistance(), top)){
-                        if ((*q).first->askState(0)) {
-                            (*q).first->changeState();
-                            nodeQueue.erase((*q).first);
-                            nodeQueue.insert((*q).first);
-                        }
-                    }
+              //Node* top = dealMap.top();
+              //dealMap.pop();
+              Node* top = *nodeQueue.begin();
+              nodeQueue.erase(nodeQueue.begin());
+              
+              std::vector<std::pair<Node*, double> >::iterator q = top->neighbor.begin();
+              for (; q != top->neighbor.end(); q++) {
+                double newDis = (*q).second + top->getDistance();
+                int compRes = (*q).first->compDistance(newDis, top);
+                if (compRes){
+                  if ((*q).first->askState(0)) {
+                    (*q).first->changeState();
+                    (*q).first->setDistance(newDis);
+                    (*q).first->setFather(top);
+                    nodeQueue.insert((*q).first);
+                  }
+                  else if ((*q).first->askState(1) && compRes == 2) {
+                    nodeQueue.erase((*q).first);
+                    (*q).first->setDistance(newDis);
+                    (*q).first->setFather(top);
+                    nodeQueue.insert((*q).first);
+                  }
                 }
-                
-                if (top->getName() != start) {
-                    shortestMap[start*size + top->getName()] = top->getDistance();
-                    directMap[start*size + top->getName()] = top->whoisYourFather()->getName();
-                }
-                top->changeState();
-                reachedNode.push_back(top);
-            }
+              }
+              
+              if (top->getName() != start) {
+                shortestMap[start*size + top->getName()] = top->getDistance();
+                directMap[start*size + top->getName()] = top->whoisYourFather()->getName();
+              }
+              top->changeState();
+              reachedNode.push_back(top);
+            } 
             
             //return GetPathVector(start, reachedNode);
             //about this part see the paper of betweenness for more details
@@ -448,60 +454,64 @@ private:
             nodeQueue.insert(startNode);
             //dealMap.push(startNode);
             
-            vector<Node*> reachedNode;
+            set<Node*, compareNode> reachedNode;
             
             while (!nodeQueue.empty()) {
-                //Node* top = dealMap.top();
-                //dealMap.pop();
-                Node* top = *nodeQueue.begin();
-                nodeQueue.erase(nodeQueue.begin());
-                
-                std::vector<std::pair<Node*, double> >::iterator q = top->neighbor.begin();
-                for (; q != top->neighbor.end(); q++) {
-                    if ((*q).first->askState(2)) continue;
-                    double tempDist = (*q).first->getDistance();
-                    if ((*q).first->setDistance((*q).second + top->getDistance(), top)){
-                        if ((*q).first->askState(0)) {
-                            (*q).first->changeState();
-                            nodeQueue.erase((*q).first);
-                            nodeQueue.insert((*q).first);
-                        }
-                    }
+              //Node* top = dealMap.top();
+              //dealMap.pop();
+              Node* top = *nodeQueue.begin();
+              nodeQueue.erase(nodeQueue.begin());
+              
+              std::vector<std::pair<Node*, double> >::iterator q = top->neighbor.begin();
+              for (; q != top->neighbor.end(); q++) {
+                int compRes = (*q).first->compDistance((*q).second + top->getDistance(), top);
+                double newDis = (*q).second + top->getDistance();
+                if (compRes){
+                  if ((*q).first->askState(0)) {
+                    (*q).first->changeState();
+                    (*q).first->setDistance(newDis);
+                    (*q).first->shortestFatherList.clear();
+                    (*q).first->shortestFatherList.push_back(top);
+                    (*q).first->setFather(top);
+                    nodeQueue.insert((*q).first);
+                    omega[(*q).first->getName()] = omega[top->getName()];
+                  }
+                  else if ((*q).first->askState(1) && compRes == 2) {
+                    nodeQueue.erase((*q).first);
+                    (*q).first->setDistance(newDis);
+                    (*q).first->shortestFatherList.clear();
+                    (*q).first->shortestFatherList.push_back(top);
+                    (*q).first->setFather(top);
+                    nodeQueue.insert((*q).first);
+                    omega[(*q).first->getName()] = omega[top->getName()];
+                  }
+                  else if (compRes == 1) {
+                    (*q).first->shortestFatherList.push_back(top);
+                    omega[(*q).first->getName()] += omega[top->getName()];
+                  }
                 }
+              }
                 
                 if (top->getName() != start) {
                     shortestMap[start*size + top->getName()] = top->getDistance();
                     directMap[start*size + top->getName()] = top->whoisYourFather()->getName();
                 }
                 top->changeState();
-                reachedNode.push_back(top);
-          }
-            
-            for (long int i = reachedNode.size() - 1; i >= 0; i--) {
-                for (long int j = 0; j < reachedNode[i]->shortestFatherList.size(); j++) {
-                    reachedNode[i]->shortestFatherList[j]->shortestChildList.push_back(reachedNode[i]);
-                }
-            }//record the child of it for further calculation
-            
-            for (long int i = 0; i < reachedNode.size(); i++) {
-                long int v = reachedNode[i]->getName();
-                for (long int j = 0; j < reachedNode[i]->shortestChildList.size(); j++) {
-                    long int w = reachedNode[i]->shortestChildList[j]->getName();
-                    omega[w] = omega[w] + omega[v];
-                }
+                reachedNode.insert(top);
             }
             
-            for (long int i = reachedNode.size() - 1; i >= 0; i--) {
-                long int w = reachedNode[i]->getName();
-                for (long int j = 0; j < reachedNode[i]->shortestFatherList.size(); j++) {
-                    long int v = reachedNode[i]->shortestFatherList[j]->getName();
-			if (omega[w] == 0) lamda[v] = lamda[v] + (omega[v]*(1 + lamda[w]));
-                    else lamda[v] = lamda[v] + (omega[v]/omega[w]*(1 + lamda[w]));
-                }
-                if (w != start) {
-                    betweenness[w] = betweenness[w] + lamda[w];// /((size-1)*(size-2))
-                }
+            for (std::set<Node*>::reverse_iterator i = reachedNode.rbegin(); i != reachedNode.rend(); i++) {
+              long int w = (*i)->getName();
+              // cout << (*i)->getDistance() << " ";
+              for (long unsigned int j = 0; j < (*i)->shortestFatherList.size(); j++) {
+                long int v = (*i)->shortestFatherList[j]->getName();
+                lamda[v] += (omega[v]/omega[w]*(1 + lamda[w]));
+              }
+              if (w != start) {
+                betweenness[w] = betweenness[w] + lamda[w];// /((size-1)*(size-2))
+              }
             }
+            // cout << endl;
             
             delete [] omega;
             delete [] lamda;
@@ -527,7 +537,7 @@ private:
     }//calculate one single path
     
     long int calculatePath(){
-        long int length = 0;
+        long unsigned int length = 0;
         for (long int i = 0; i < size; i++) {
             for (long int j = 0; j < size; j++) {
                 if (i != j) {
@@ -622,7 +632,6 @@ public:
     
     void triangleArmadillo(){
         triangles = 0;
-        
         arma::mat I(distMap,size,size,true);
         
         arma::umat comparePath = (I != metric_global_DISTMAX);
@@ -634,6 +643,7 @@ public:
             
             arma::umat result = (comparePath % needPath);
             triangles+=(long int)arma::accu(result);
+            //cout << triangles << endl;
         }
         triangles/=3;//calculation triangle in matrix form.
     }
@@ -648,6 +658,7 @@ public:
                     Dijiastra(size, distMap, shortestMap, directMap);
                 }
                 else Floyd(size, distMap, shortestMap, directMap);
+                //Floyd(size, distMap, shortestMap, 1);
                 break;//only shortest path
                 
             case 1://betweenness
@@ -683,12 +694,14 @@ public:
                 if (isGraphSym) {
                     triangles*=2;
                 }
+                // cout << triangles << endl;
                 if (taskType != 1) {
                     Dijiastra(betweenness, size, distMap, shortestMap, directMap);
                 }
                 FloydBasedBetweenness(betweenness, size, distMap, shortestMap, directMap);
                 triangleArmadillo();
-
+                // cout << triangles << endl;
+                
             default:
                 break;
         }
@@ -716,8 +729,7 @@ public:
     
     double* getBetweenness(){
         if (betweenness == NULL) {
-            //cout << "Alloc Error" << endl;
-            return NULL;
+          //cout << "Alloc Error" << endl;
         }
         return betweenness;
     }//return the betweenness
@@ -813,14 +825,14 @@ SEXP metric_global_shortestDetails(NumericMatrix disMap){
     }
     
     metric_global_GraphTools tempx(inputMatrix, size, 3);
-    long int length = tempx.shortestPathDetails();
+    long unsigned int length = tempx.shortestPathDetails();
     
     vector<vector<vector<long int> > > result = tempx.getPath();
     
     NumericVector out=NumericVector(Dimension(length,size,size));
     for (int i = 0; i < size; i++) {
         for (int j = 0; j < size; j++) {
-            for (int k = 0; k < length; k++) {
+            for (long unsigned int k = 0; k < length; k++) {
                 if (k < result[i][j].size()) {
                     out[i*size*length + j *length + k] = result[i][j][k];
                 }
